#include <Arduino.h>
#include <SPI.h>
#include <Pixy2.h>
#include <Wire.h> // Incluye la biblioteca Wire para la comunicación I2C
#include <Pixy2I2C.h> // Incluye la biblioteca Pixy2I2C.h
#include "bluetooth.h" 
#include "motores.h"

#define X_CENTER 160 // Valor central del eje X del campo de visión de la Pixy2
#define MAX_SPEED 100

Pixy2I2C pixy; // Creamos un objeto Pixy2

void pulsarArriba(); //Aquí se declaran las funciones que corresponden a cada uno de los botones de la aplicación que vamos a usar para controlar los robots
void soltarArriba();
void pulsarDerecha();
void soltarDerecha();
void pulsarIzquierda();
void soltarIzquierda();
void pulsarAbajo();
void soltarAbajo();

bool modo_manual=true;
int vel =100; //velocidad del robot, de 33 a 100
int vel_der=0;
int vel_iz=0;
int dir =0;
 int compensacion;
//VELOCIDAD MINIMA ACEPTABLE 27
//VELOCIDAD MINIMA CON PULSOS 20
//VELOCIDAD MINIMA SIN PULSOS 33
float Kp = 35;                 // Constante proporcional
float Ti = 6000;                // Tiempo integral
float Td = 5000e-6;            // Tiempo derivativo

float direccion;           // Dirección que se va a tomar en el PD
float direccion_anterior;  // Dirección anterior de la lectura de los sensores activos

float error;           // La variable error es la parte proporcional
float error_anterior;  // La variable error_anterior es la parte derivativa
float integral_error;  // La variable integral_error es la que almacena la integral del error
float max_integral = 1000;

//#define SDA_PIN 23 // Define el pin SDA que se utilizará
//#define SCL_PIN 22 // Define el pin SCL que se utilizará

void setup() {

configurarDispositivos(); //Importamos la configuración de la librería
setupBluetooth();  // Esta línea corresponde a la inicialización de la comunicación bluetooth
  Serial.begin(115200);
  Serial.print("Starting...\n");
  Wire.begin(23, 22); 


  detenerse(); // Asegurarse de que los motores estén detenidos al inicio

  pixy.init();
  pixy.setLamp(1, 1);
  pixy.changeProg("color_connected_components"); // Cambiar al programa de seguimiento de color
}





void loop() {  
  
   loopBluetooth(); 

   if(modo_manual==false)
   {
  int16_t blocks;
  int32_t error;

  blocks = pixy.ccc.getBlocks();
  
  if (blocks) {
    // Si se detecta al menos un bloque
    int16_t posX = pixy.ccc.blocks[0].m_x;
    //Serial.print("Linea detectada");
    
    if (posX > 125 && posX < 175)
    {
      // compensacion = map(posX, 100, 250, 0, 55); // mapea la posición de la linea roja entre 0 y 55 que son los valores de velocidad con los que vamos a jugar para corregir la posición del miniAGV
       //if(posX> 150)
       //{
        //compensacion =-compensacion;
       //}
      //avanzar(200+ compensacion,200-compensacion);
      avanzar(vel);
      delay(10);
            Serial.print("Linea centrada: ");
            Serial.println(posX);
    }
    else if (posX > 175 && posX < 210) {
      // Si el objeto está más a la derecha, girar hacia la derecha
      //avanzar(0, 255); // Girar hacia la derecha
      vel_iz = map(posX, 175, 210, vel, 0);
     vel_der=vel;
      girar(vel_iz, vel_der);
      delay(10);
      Serial.print("Linea detectada a la derecha: ");
            Serial.println(posX);
    } 
        else if (posX > 210) {
      // Si el objeto está más a la derecha, girar hacia la derecha
      //avanzar(0, 255); // Girar hacia la derecha
      pivotar_derecha(50);
    } 
    else if (posX < 125 && posX >90) {
      // Si el objeto está más a la izquierda, girar hacia la izquierda
      Serial.print("Linea detectada a la izquierda: ");
            Serial.println(posX);
     // avanzar(255, 0); // Girar hacia la izquierda
   vel_der = map(posX, 125, 90, vel, 0);
     vel_iz=vel;
     girar(vel_iz, vel_der);
      delay(10);
    }  else if (posX < 90) {
pivotar_izquierda(50);
    } ///else {
      // Si el objeto está centrado, avanzar recto
      //avanzar(MAX_SPEED, MAX_SPEED);
    //}
  } else {
    // Si no se detectan bloques, detenerse
    detenerse();
  }

   } 


}




void mantener_posicion()
{

  detenerse();
}
void pulsarArriba() //corresponde al botón de la flecha hacia arriba
{
 avanzar(vel); 
}

void soltarArriba() // Aquí debeis declarar qué ocurre cuando se suelta el botón de la flecha hacia arriba
{
detenerse();
}



void pulsarDerecha()//corresponde al botón de la flecha hacia la derecha
{
 pivotar_derecha(100);
}

void soltarDerecha() //Aquí debeis declarar qué ocurre cuando se suelta el botón de la flecha hacia la derecha
{
    detenerse();
    Serial.println("soltar derecha");
}


void pulsarIzquierda()//corresponde al botón de la flecha hacia la izquierda
{
  pivotar_izquierda(100);
}

void soltarIzquierda() //Aquí debeis declarar qué ocurre cuando se suelta el botón de la flecha hacia la izquierda
{
    detenerse();
}


void pulsarAbajo()//corresponde al botón de la flecha hacia abajo
{
 retroceder(vel);
}

void soltarAbajo() //Aquí debeis declarar qué ocurre cuando se suelta el botón de la flecha hacia abajo
{
detenerse();
}

void cambio_modo()
{
 if(modo_manual == true)
 {
  vel =33;
  modo_manual=false;
  Serial.println("modo automatico");
 } else{
  modo_manual=true;
  Serial.println("modo manual");
 }
}

void velocidad_lenta()
{
  vel=33;
  Serial.println(vel);
}
void velocidad_media()
{
  vel=66;
  Serial.println(vel);
}
void velocidad_rapida()
{
  vel=100;
  Serial.println(vel);
}
